/*
 * Copyright (c) 2001 Massachusetts Institute of Technology.
 *
 * This file is part of bgptools. 
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `COPYING' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file AUTHORS which should have accompanied this software 
 * for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the AUTHORS file.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


/*
 *     Nick Feamster (feamster@lcs.mit.edu)
 *     MIT Lab for Computer Science
 *     November 2001
 */

#include "db.h"
#ifdef __USE_SQLPLUS__

db::db(char *db, cchar *host=DB_HOST, cchar *user="", cchar *passwd="") {
  this->db_file = db;
  this->host = host;
  fprintf(stderr, "connection to %s at %s...", db, host);
  this->c = new Connection(db, host, user, passwd);
  fprintf(stderr, "succeeded.\n");
}


void
db::add_update_file(char *filename, unsigned long start_ep,
		    unsigned long end_ep) {
  FILE *fp;
  update upd;

  fp = fopen(filename, "rb");
  if (fp == NULL) { fprintf(stderr, "error: could not open %s\n", filename); };
  

  while(!feof(fp)) {

    read_record(fp, &upd);
    if (upd.prefix_) {
        if ((upd.time > start_ep && upd.time < end_ep) || (!start_ep))
	  this->add_update(&upd);
    }

    if (upd.prefix_) { free(upd.prefix_); upd.prefix_=NULL; }
//      if (upd.prefix) { free(upd.prefix); upd.prefix=NULL; }


  }
  fclose(fp);
  
  
}

void
db::add_update(struct update *upd, int file=0) {
  Query query = c->query();

  char ad_char[2];
  char orig_char[3];

  char dest_ip [sizeof(".xxx.xxx.xxx.xxx")];
  char next_hop[sizeof(".xxx.xxx.xxx.xxx")];
  char prefix[sizeof(".xxx.xxx.xxx.xxx/xx")];

  char datestr[11];
  char timestr[9];
  char querystr[1024];
 
  ad_char[BGP_ADV] = 'A';
  ad_char[BGP_WITHDRAWAL] = 'W';

  orig_char[IGP] = 'I';
  orig_char[EGP] = 'E';
  orig_char[INCOMPLETE] = '?';


  strncpy(dest_ip, (char*)intoa(upd->next_hop), sizeof(dest_ip));
  strncpy(next_hop, (char*)intoa(upd->next_hop), sizeof(next_hop));

  struct tm *tmp;

  tmp = localtime((const time_t*)(&upd->time));
  tmp->tm_year+=1900;
  tmp->tm_mon++;

  sprintf(datestr, "%.4d-%.2d-%.2d", tmp->tm_year,
	  tmp->tm_mon, tmp->tm_mday);
  datestr[10] = '\0';
  sprintf(timestr, "%.2d:%.2d:%.2d", tmp->tm_hour, tmp->tm_min,
  	  tmp->tm_sec);
  timestr[8] = '\0';


  for (int i=0; i<upd->num_prefixes; i++) {
    strncpy(prefix, (char*)intoa(upd->prefix_[i].pfx), sizeof(prefix));
    sprintf(prefix, "%s/%d", prefix, upd->prefix_[i].len);

//      if ((upd->ad_type&1) == BGP_ADV) {
      sprintf(querystr, "insert into %s.%s values (%d, '%c',%d,%d,'%u', %d,'%s',%d,'%d',%d,%d,'%c');",
	      this->db_file, UPDATE_TABLE,
	      upd->time,
	      'A', upd->src_as, upd->dst_as, upd->prefix_[i].pfx,
	      upd->prefix_[i].len, upd->as_path, upd->origin_as,
	      upd->next_hop, upd->med, upd->localpref, orig_char[upd->origin]);  
     
//      } else if ((upd->ad_type&1) == BGP_WITHDRAWAL) {
//      }

    query << querystr;
//      cout << query.preview() << endl;
    query.execute();
  }

  for (int i=0; i<upd->num_prefixes_wd; i++) {
    sprintf(querystr,
	    "insert into %s.%s values (%d,'%c',%d,%d,'%u',%d,NULL,NULL,NULL,NULL,NULL,NULL);",
	    this->db_file, UPDATE_TABLE,
	    upd->time,
	    'W', upd->src_as, upd->dst_as,
	    upd->prefix_wd_[i].pfx,
	    upd->prefix_wd_[i].len);
    
      query << querystr;
//        cout << query.preview() << endl;
      query.execute();

  }



}

db::~db() {
  delete c;
}


#endif
